-- ===================================================
-- Constants
-- ===================================================

local ADDON_NAME, Mocksamdi = ...;
local ADDON_VERSION = Mocksamdi.VERSION;

local E_ADDON_LOADED = "ADDON_LOADED";  -- fires when saved variables are loaded
local E_PLAYER_DEAD = "PLAYER_DEAD";  -- fires whern player dies
local E_PLAYER_UNGHOST = "PLAYER_UNGHOST" -- fires when the player becomes alive again
local E_RESURRECT_REQUEST = "RESURRECT_REQUEST" -- fires when an ally resurrects the player
local E_UNIT_AURA = "UNIT_AURA" -- fires when a buff or debuff is added or removed from the player

local ALL_EVENTS = {
	-- E_RESURRECT_REQUEST, -- unused, future
	-- E_PLAYER_UNGHOST, -- This is registered on the fly when PLAYER_DEAD fires and unregistered when fired.
	-- E_UNIT_AURA, -- This is registered on the fly when PLAYER_DEAD fires and unregisted when PLAYER_UNGHOST is fired
  E_ADDON_LOADED,
	E_PLAYER_DEAD
}

-- ===================================================
-- Variables
-- ===================================================

local eventFrame = CreateFrame("FRAME"); -- respond to events

-- ===================================================
-- Local Functions
-- ===================================================

-- Check if player is in Nazmir
local isInNazmir = function ()
	-- TODO: He taunts in Nazmir already so disable when there
	local zoneName = GetZoneText();
	Mocksamdi:printDebug("Is in " .. zoneName);
	return zoneName == "Nazmir";
end

-- Check if we should play the mock sound or not
local shouldPlayMock = function()
	local inInstance, instanceType = IsInInstance();

	-- don't double dip
	if isInNazmir() then
		Mocksamdi:printDebug("Is in Nazmir");
		return false;
	end

	Mocksamdi:printDebug("Checking if Play. inInstance: " .. (inInstance and 'true' or 'false') .. " instanceType: " .. (instanceType and instanceType or 'nil'));

	-- if we arent in an instence or known type then we're good to play it
	if not inInstance or instanceType == "none" or instanceType == nil then
		return true;
	end

	-- bg
	if instanceType == "pvp" and MocksamdiInBattleground then
		return true;
	end

	-- arena
	if instanceType == "arena" and MocksamdiInArena then
		return true;
	end

	-- dungeon
	if instanceType == "party" and MocksamdiInDungeon then
		return true;
	end

	-- raid
	if instanceType == "raid" and MocksamdiInRaid then
		return true;
	end

	Mocksamdi:printDebug("Unrecognised instance type " .. (instanceType and instanceType or 'nil'));
	return true;
end

-- handle addon load event
local handleAddonLoadEvent = function()
	eventFrame:UnregisterEvent(E_ADDON_LOADED) -- cleanup

	-- set defaults if needed
	if MocksamdiShowStartup == nil then MocksamdiShowStartup = true end
	if MocksamdiShowImage == nil then MocksamdiShowImage = true end
	if MocksamdiPlayAudio == nil then MocksamdiPlayAudio = true end
	if MocksamdiInRaid == nil then MocksamdiInRaid = true end
	if MocksamdiInDungeon == nil then MocksamdiInDungeon = true end
	if MocksamdiInArena == nil then MocksamdiInArena = true end
	if MocksamdiInBattleground == nil then MocksamdiInBattleground = true end
	if MocksamdiDebug == nil then MocksamdiDebug = false end

	-- debug variable values
	Mocksamdi:printDebug("Saved Vars:")
	Mocksamdi:printDebug("- MocksamdiShowStartup: " .. (MocksamdiShowStartup and 'true' or 'false'));
	Mocksamdi:printDebug("- MocksamdiShowImage: " .. (MocksamdiShowImage and 'true' or 'false'));
	Mocksamdi:printDebug("- MocksamdiPlayAudio: " .. (MocksamdiPlayAudio and 'true' or 'false'));
	Mocksamdi:printDebug("- MocksamdiInRaid: " .. (MocksamdiInRaid and 'true' or 'false'));
	Mocksamdi:printDebug("- MocksamdiInDungeon: " .. (MocksamdiInDungeon and 'true' or 'false'));
	Mocksamdi:printDebug("- MocksamdiInArena: " .. (MocksamdiInArena and 'true' or 'false'));
	Mocksamdi:printDebug("- MocksamdiInBattleground: " .. (MocksamdiInBattleground and 'true' or 'false'));

	-- Init stuff
	Mocksamdi:initModel();
	Mocksamdi:initConfig();

	-- Check show startup message
	if MocksamdiShowStartup then
		print("Loaded |cff00E5EE" .. ADDON_NAME .. "|r at version |cffffcc00" .. ADDON_VERSION .. "|r")
	end
end

-- handle death event
local handleDeathEvent = function ()
	Mocksamdi:printDebug("Player is dead");
	eventFrame:RegisterEvent(E_UNIT_AURA); -- listen for auras so we can catch when the player catches Ghost
	eventFrame:RegisterEvent(E_PLAYER_UNGHOST); -- listen for alive event. if the player resurects we need to clean up unit aura
end

local handleAliveEvent = function ()
	Mocksamdi:printDebug("Player is alive again");
	eventFrame:UnregisterEvent(E_UNIT_AURA); -- cleanup unit aura until the next death
	eventFrame:UnregisterEvent(E_PLAYER_UNGHOST); -- cleanup unghost until the next death
end

-- hangle the ghost event
local handleGhostEvent = function ()
	-- find out if the player catches ghost
	local foundGhost = false;
	for i=1, 40 do
		local debuffName = UnitDebuff("player", i);
		if debuffName == "Ghost" then
			foundGhost = true;
			break;
		end
	end

	Mocksamdi:printDebug("Player is a ghost: " .. (foundGhost and 'true' or 'false'));

	if not foundGhost then
		return;
	end

	eventFrame:UnregisterEvent(E_UNIT_AURA); -- cleanup unit aura until the next death

	if shouldPlayMock() then
		Mocksamdi:playMockSound();
		Mocksamdi:showModel();
	end
end

-- ===================================================
-- Global Functions
-- ===================================================

function Mocksamdi:init ()
	-- register events
	for i=1, table.getn(ALL_EVENTS) do
		Mocksamdi:printDebug("Register Event: " .. ALL_EVENTS[i]);
		eventFrame:RegisterEvent(ALL_EVENTS[i]);
	end

	eventFrame:SetScript("OnEvent",
		function (self, event, ...)
			local arg1, arg2, arg3 = ...
			-- Debug all events (except other plugins loading)
			if event ~= E_ADDON_LOADED or arg1 == ADDON_NAME then
				Mocksamdi:printDebug("Event: " .. event .. " " .. (arg1 and arg1 or 'nil') .. " " .. (arg2 and arg2 or 'nil') .. " " .. (arg3 and arg3 or 'nil'));
			end

			-- Startup
			if event == E_ADDON_LOADED and arg1 == ADDON_NAME then
				handleAddonLoadEvent();
			end

			-- Check if player died
			if event == E_PLAYER_DEAD then
				handleDeathEvent();
			end

			-- Check if player is no longer a ghost
			if event == E_PLAYER_UNGHOST then
				handleAliveEvent();
			end

			-- Check if player got the ghost aura
			if event == E_UNIT_AURA and arg1 == "player" then
				handleGhostEvent();
			end
		end
	);
	eventFrame:Hide();
end
